@@ -9,11 +9,11 @@
 __author__      = ["Claudio D'Onofrio"]
 __credits__     = "ICOS Carbon Portal"
 __license__     = "GPL-3.0"
-__version__     = "0.1.7"
+__version__     = "0.1.8"
 __maintainer__  = "ICOS Carbon Portal, elaborated products team"
 __email__       = ['info@icos-cp.eu', 'claudio.donofrio@nateko.lu.se']
 __status__      = "stable"
-__date__        = "2022-05-24"
+__date__        = "2023-02-27"
 
 import os
 from warnings import warn
@@ -25,6 +25,10 @@ from icoscp import __version__ as release_version
 from icoscp.cpb import dtype
 from icoscp.cpb import metadata
 import icoscp.const as CPC
+from icoscp.cpauth.authentication import Authentication
+from icoscp.cpauth.exceptions import AuthenticationError, warn_for_authentication_bypass
+
+from json.decoder import JSONDecodeError
 
 
 class Dobj():
@@ -33,7 +37,13 @@ class Dobj():
         the method .getColumns() will return the actual data
     """
 
-    def __init__(self, digitalObject = None):
+    # Private class attribute used to bypass authentication for consecutive
+    # requests of data objects if the first authentication attempt was
+    # unsuccessful. It is only applicable to off-server data access.
+    _bypass_auth = False
+
+    def __init__(self, digitalObject = None, debug_auth: str = False,
+                 cp_auth: Authentication = None):
 
         self._dobj = None           # contains the pid
         self._colSelected = None    # 'none' -> ALL columns are returned
@@ -46,16 +56,17 @@ class Dobj():
         self._json = None           # holds the "payload" for requests
         self._islocal = None        # status if file is read from local store
                                     # if localpath + dobj is valid
-        
-        self._dobjValid = False     # -> see __set_meta()        
+
+        self._dobjValid = False     # -> see __set_meta()
         self._data = pd.DataFrame() # This holds the data pandas dataframe
                                     # persistent in the object.
         self._datapersistent = True # If True (default), data is kept persistent
                                     # in self._data. If False, force to reload
-                                    
+        self.cp_auth = cp_auth
+        self._debug_auth = debug_auth
         # this needs to be the last call within init. If dobj is provided
         # meta data is retrieved and .valid is True
-        self.dobj = digitalObject   
+        self.dobj = digitalObject
 
 
     #-----------
@@ -183,10 +194,8 @@ class Dobj():
     @property
     def citation(self):
         return self.get_citation('plain')
-# -------------------------------------------------
 
     def __str__(self):
-        
         if not self.valid:
             return ''
         
@@ -330,31 +339,84 @@ class Dobj():
             check if a local path is set and valid
             otherwise try to download from the cp server
         """
-        
+
         #assemble local file path
         folder = self.meta['specification']['format']['uri'].split('/')[-1]
         fileName = ''.join([self.dobj.split('/')[-1],'.cpb'])
-        localfile = os.path.abspath(''.join([CPC.LOCALDATA,folder,'/',fileName]))
+        # Don't set the local path for data files when debugging the
+        # authentication module.
+        local_file = str()
+        if not self._debug_auth:
+            local_file = \
+                os.path.abspath(f'{CPC.LOCALDATA}{folder}/{fileName}')
 
-        if os.path.isfile(localfile):
+        # Local access on server.
+        if os.path.isfile(local_file):
             self._islocal = True
-            with open(localfile, 'rb') as binData:
+            with open(local_file, 'rb') as binData:
                 content = binData.read()
-            # for local files, we always return ALL columns
-            self._colSelected = list(range(0,len(self.variables)))
+            # For local files, we always return all columns.
+            self._colSelected = list(range(0, len(self.variables)))
             self.__getPayload()
-
+            # Track data usage for data access on server.
+            self.__portalUse()
+        # Access through HTTP request.
         else:
             self._islocal = False
-            r = requests.post(CPC.DATA, json=self._json, stream=True)
+            response, content = None, None
+            request_url, request_headers = None, None
+            if Authentication._bypass_auth:
+                warn_for_authentication_bypass(
+                    reason=Authentication._bypass_exception
+                )
+            # User authentication not in place.
+            elif not self.cp_auth and not Dobj._bypass_auth:
+                # Try obtaining the default authentication configuration.
+                try:
+                    self.cp_auth = Authentication()
+                # Initialize the authentication process if the
+                # configuration at the default location is not set or
+                # if it is wrongly formatted.
+                except JSONDecodeError as e:
+                    try:
+                        self.cp_auth = Authentication(initialize=True)
+                    except AuthenticationError as e:
+                        warn_for_authentication_bypass(reason=e)
+                except AuthenticationError as e:
+                    warn_for_authentication_bypass(reason=e)
+                else:
+                    if Authentication._bypass_auth:
+                        warn_for_authentication_bypass(
+                            reason=Authentication._bypass_exception
+                        )
+            # Authentication was successful.
+            if self.cp_auth and self.cp_auth.valid_token:
+                request_url = CPC.SECURED_DATA
+                # The API token should have been set by now either by
+                # authentication provided as an argument,
+                # authentication retrieved from the default location,
+                # or authentication reset; thus, set the headers for
+                # the post request.
+                request_headers = {'cookie': self.cp_auth.token}
+            # Fall back to anonymous data access if all other
+            # authentication ways fail.
+            else:
+                Dobj._bypass_auth = True
+                request_url = CPC.ANONYMOUS_DATA
+            # Request data either anonymously or in an authenticated way.
+            response = requests.post(url=request_url,
+                                     json=self._json,
+                                     stream=True,
+                                     headers=request_headers)
             try:
-                r.raise_for_status()
-                content = r.content
+                response.raise_for_status()
             except requests.exceptions.HTTPError as e:
-                raise Exception(e)
-
-        #track data usage
-        self.__portalUse()
+                raise e
+            else:
+                if response.status_code == 200:
+                    content = response.content
+                    # Track usage for data access.
+                    self.__portalUse(service=request_url)
         return self.__unpackRawData(content)
 
     def __unpackRawData(self, rawData):
@@ -368,17 +430,46 @@ class Dobj():
         # make sure the list is sorted by variable name, this is how the
         # binary fileformat is built.
         columns.sort()
-        
-        
-        rows = int(self.meta['specificInfo']['nRows'])
 
+        # Patch code for timestamp conversions. To be reworked
+        # in future release.
+        def prepend_ontology(segm):
+            """Convert timestamps to a pandas date/time object."""
+            return f'{CPC.CP_META}{segm}'
+        # Stored as seconds since midnight.
+        time_formats = list(map(prepend_ontology, ['iso8601timeOfDay']))
+        # Stored as days since epoch.
+        date_formats = list(map(prepend_ontology, ['iso8601date', 'etcDate']))
+        # Stored as milliseconds since epoch.
+        datetime_formats = list(map(prepend_ontology, ['iso8601dateTime',
+                                                       'isoLikeLocalDateTime',
+                                                       'etcLocalDateTime']))
+        # Stored as integer.
+        integer_format = list(map(prepend_ontology, ['int32']))
+        def is_time(value_format): return value_format in time_formats
+        def is_date(value_format): return value_format in date_formats
+        def is_datetime(value_format): return value_format in datetime_formats
+        def is_int(value_format): return value_format in integer_format
+        fmt = self.variables.sort_values('name')['format'].tolist()
+        rows = int(self.meta['specificInfo']['nRows'])
         try:
-            for idx,col in enumerate(self._colSelected):
+            for idx, col in enumerate(self._colSelected):
                 lst = list(data[idx*rows:(idx+1)*rows])
                 if self._colSchema[col] == 'CHAR':
-                    #convert UTF-16 , this is often used in "Flag" columns
+                    # Convert UTF-16, which is often used in "Flag"
+                    # columns.
                     lst = [chr(i) for i in lst]
-                df[columns[col]] = lst
+                elif is_datetime(fmt[col]):
+                    lst = pd.to_datetime(lst, unit='ms')
+                elif is_date(fmt[col]):
+                    lst = pd.to_datetime(lst, unit='D')
+                elif is_time(fmt[col]):
+                    lst = pd.to_datetime(lst, unit='s')
+                elif is_int(fmt[col]):
+                    # Keep years and other integers as integers.
+                    pass
+                df = pd.concat([df, pd.Series(lst).rename(columns[col])],
+                               axis=1)
                 
         except Exception as e:
             raise Exception('_unpackRawData')
@@ -416,21 +507,26 @@ class Dobj():
         return df
 
     # -------------------------------------------------
-    def __portalUse(self):
-
-        """ private function to track data usage """
-
-        counter = {'BinaryFileDownload':{
-        'params':{
-            'objId':self._dobj,
-            'columns': self.colNames,
-            'library':__name__,
-            'version':release_version,  # from '__init__.py'.
-            'internal': str(self._islocal)}
-            }
+    def __portalUse(self, service: str = None) -> None:
+        """Private function to track data usage."""
+        counter = {
+            'BinaryFileDownload':
+                {
+                    'params':
+                        {
+                            'objId': self._dobj,
+                            'columns': self.colNames,
+                            'library': __name__,
+                            # Retrieved from __init__.py
+                            'version': release_version,
+                            'internal': str(self._islocal),
+                            'service': service,
+                        },
+                }
         }
         server = 'https://cpauth.icos-cp.eu/logs/portaluse'
         requests.post(server, json=counter)
+        return
 
     # -------------------------------------------------
     def __setColumns(self, columns=None):
