@@ -8,11 +8,12 @@
 
 __credits__     = "ICOS Carbon Portal"
 __license__     = "GPL-3.0"
-__version__     = "0.1.2"
+__version__     = "0.1.4"
 __maintainer__  = "ICOS Carbon Portal, elaborated products team"
 __email__       = ['info@icos-cp.eu']
 __status__      = "release"
-__date__        = "2022-02-11"
+__date__        = "2023-01-18"
+__lastchange__  = ["Zois Zogopoulos"]
 #################################################################################
 
 #Import modules
@@ -24,6 +25,7 @@ import json
 import xarray as xr
 import icoscp.const as CPC
 from icoscp.stilt import timefuncs as tf
+from icoscp.sparql import sparqls, runsparql
 from icoscp import __version__ as release_version
 ##############################################################################
 
@@ -63,6 +65,8 @@ class StiltStation():
         self.icos = None
         self.years = None
         self.geoinfo = None
+        self.dobjs_list = None               # Store a list of associated dobjs
+        self.dobjs_valid = False        # If True, dobjs sparql query already executed        
 
         self._set(st_dict)
 
@@ -85,7 +89,7 @@ class StiltStation():
 
     def __str__(self):
         # by default called when a an 'object' is printed
-        
+
         out = {'id:': self.id,
                'name:': self.name,
                'lat:': self.lat,
@@ -95,7 +99,7 @@ class StiltStation():
         return json.dumps(out)
 
     #----------------------------------------------------------------------------------------------------------
-    def get_ts(self, start_date, end_date, hours=[], columns=''):
+    def get_ts(self, start_date, end_date, hours=None, columns=None):
         """
         STILT concentration time series for a given time period,
         with optional selection of specific hours and columns.
@@ -119,8 +123,8 @@ class StiltStation():
                         hours = ["10", "10:00", 10] returns timeslot 9
 
         columns : TYPE, optional
-            Valid entries are "default", "co2", "co", "rn", "wind", "latlon", "all"
-            default (or empty) will return
+            Valid entries are "default", "co2", "co", "rn", "wind", "latlon", "all".
+            'default', empty, or None will return:
             ["isodate","co2.stilt","co2.bio","co2.fuel","co2.cement","co2.background"]
             A full description of the 'columns' can be found at
             https://icos-carbon-portal.github.io/pylib/modules/#stilt
@@ -195,8 +199,10 @@ class StiltStation():
                 #Get response in json-format and read it in to a numpy array:
                 output=np.asarray(response.json())
 
-                #Convert numpy array with STILT results to a pandas dataframe:
-                df = pd.DataFrame(output[:,:], columns=eval(columns))
+                #Convert numpy array with STILT results to a pandas dataframe
+                cols = columns[1:-1].replace('"','').replace(' ','')
+                cols = list(cols.split(','))
+                df = pd.DataFrame(output[:,:], columns=cols)
 
                 #Replace 'null'-values with numpy NaN-values:
                 df = df.replace('null',np.NaN)
@@ -225,7 +231,7 @@ class StiltStation():
         return df
     #----------------------------------------------------------------------------------------------------------
 
-    def get_fp(self, start_date, end_date, hours=[]):
+    def get_fp(self, start_date, end_date, hours=None):
         """
         STILT footprints for a given time period,
         with optional selection of specific hours.
@@ -291,7 +297,11 @@ class StiltStation():
         #Concatenate xarrays on time axis:
         fp = xr.open_mfdataset(fp_files, combine='by_coords',
                                data_vars='minimal', coords='minimal',
-                               compat='override', parallel=True)
+                               compat='override', parallel=True,
+                               decode_cf=False)
+
+        # now check for CF compatibility
+        fp = xr.decode_cf(fp)
 
         #Format time attributes:
         fp.time.attrs["standard_name"] = "time"
@@ -314,7 +324,7 @@ class StiltStation():
         """
         Please do use this function with caution. Only very expirienced user
         should load raw data.
-    
+
         Parameters
         ----------
         start_date : STR
@@ -324,7 +334,7 @@ class StiltStation():
         cols : LIST[STR]
             A list of valid column names. You can retrieve the full list
             with _raw_clumn_names.
-    
+
         Returns
         -------
         columns : Pandas DataFrame
@@ -333,9 +343,9 @@ class StiltStation():
         #Convert date-strings to date objs:
         s_date = tf.parse(start_date).strftime('%Y-%m-%d')
         e_date = tf.parse(end_date).strftime('%Y-%m-%d')
-        
+
         # validate column names:
-            
+
         # make sure isodate is in the request
         if 'isodate' not in cols:
             cols.append('isodate')
@@ -343,25 +353,27 @@ class StiltStation():
         if len(columns) <= 1:
             return False
         # provide double quotes in the list
-        columns = json.dumps(columns) 
-    
+        columns = json.dumps(columns)
+
         # Check input parameters:
         if e_date < s_date:
             return False
-            
+
         # create http header and payload:
-            
-        headers = {'Content-Type': 'application/json', 'Accept-Charset': 'UTF-8'}        
+
+        headers = {'Content-Type': 'application/json', 'Accept-Charset': 'UTF-8'}
         data = '{"columns": '+ str(columns) + ',"fromDate": "'+s_date+'", "toDate": "'+e_date+'", "stationId": "'+self.id+'"}'
         response = requests.post(CPC.STILTRAW, headers=headers, data=data)
-        
+
         if response.status_code != 500:
 
             #Get response in json-format and read it in to a numpy array:
             output=np.asarray(response.json())
 
-            #Convert numpy array with STILT results to a pandas dataframe:
-            df = pd.DataFrame(output[:,:], columns=eval(columns))
+            #Convert numpy array with STILT results to a pandas dataframe
+            cols = columns[1:-1].replace('"','').replace(' ','')
+            cols = list(cols.split(','))
+            df = pd.DataFrame(output[:,:], columns=cols)
 
             #Replace 'null'-values with numpy NaN-values:
             df = df.replace('null',np.NaN)
@@ -375,19 +387,50 @@ class StiltStation():
             #Set 'date'-column as index:
             df.set_index(['date'],inplace=True)
 
-    
+
         # track data usage
         self.__portalUse('timeseries')
         #Return dataframe:
         return df
-    
-    #Function that checks the selection of columns that are to be
-    #returned with the STILT timeseries model output:
-    def __columns(self, cols):
 
-        # make parameters case insensitive
-        cols = cols.lower()
+    def get_dobj_list(self):
+        """
+        If the stiltstation has a corresponding ICOS station
+        this function will return a dictionary filled with corresponding
+        dataobjects PID's. A sparql query is executed with ICOS Station id
+        and the sampling height.
+
+        Returns
+        -------
+        DICT
+            A dictionary with the following keys:
+            [dobj,hasNextVersion,spec,fileName,size,submTime,timeStart,timeEnd]
+
+        """
         
+        if not self.valid:
+            return
+        
+        if not self.dobjs_valid and self.icos:
+            # add corresponding data object with observations
+            query = sparqls.dobj_for_samplingheight(\
+                            self.icos['stationId'], \
+                            self.icos['SamplingHeight'])
+            df = runsparql.RunSparql(query, 'pandas').run()
+            
+            self.dobjs_list = df.to_dict('records')
+            self.dobjs_valid = True        
+        
+        return self.dobjs_list
+        
+        
+    def __columns(self, cols):
+        #Function that checks the selection of columns that are to be
+        #returned with the STILT timeseries model output:
+        if cols:
+            # Convert user-specified columns to lower case.
+            cols = cols.lower()
+
         # check for a valid entry. If not...return default
         valid = ["default", "co2", "co", "rn", "wind", "latlon", "all"]
         if cols not in valid:
@@ -395,7 +438,7 @@ class StiltStation():
 
         #Check columns-input:
         if cols=='default':
-            columns = ('["isodate","co2.stilt","co2.bio","co2.fuel","co2.cement",'+ 
+            columns = ('["isodate","co2.stilt","co2.bio","co2.fuel","co2.cement",'+
                        '"co2.background"]')
 
         elif cols=='co2':
@@ -466,14 +509,14 @@ class StiltStation():
                 'internal': True}}}
         server = 'https://cpauth.icos-cp.eu/logs/portaluse'
         requests.post(server, json=counter)
-        
+
     def _raw_column_names(self):
         '''
         The STILT model calculates many different variables. We provide
         sensible groups for users, see the documentationfor .get_ts()
         This function returns an exhaustive list for all columns, approximately
         600, which can be used in conjunction get_raw()
-        
+
 
         Returns
         -------
